@@ -1,6 +1,13 @@
 # perldoc SIG.pm for documentation.
 # Specs: RFC 2535 section 4
-# $Id: SIG.pm 318 2005-05-30 16:36:52Z olaf $
+# $Id: SIG.pm 998 2012-06-28 08:52:42Z willem $
+
+BEGIN {
+	%NET_DNS_SEC_SIG_BACKUP = %main::SIG if $^V lt v5.14;
+}
+UNITCHECK {
+	%main::SIG = %NET_DNS_SEC_SIG_BACKUP if $^V lt v5.14;
+}
 
 package Net::DNS::RR::SIG;
 
@@ -18,7 +25,7 @@ use File::Basename;
 use MIME::Base64;
 use Math::BigInt;
 use Time::Local;
-use Digest::SHA1 qw (sha1);
+use Digest::SHA qw (sha1);
 
 
 
@@ -33,7 +40,7 @@ use Digest::SHA1 qw (sha1);
 
 require Exporter;
 
-$VERSION = do { my @r=(q$Revision: 318 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
+$VERSION = do { my @r=(q$Revision: 998 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 @ISA = qw (
 	   Exporter
 	 Net::DNS::RR
@@ -82,7 +89,7 @@ sub new {
 	$self->{"keytag"}=unpack("n",substr($$data,$offsettokeytag,2));
 	my($signame,$sigoffset) = Net::DNS::Packet::dn_expand
 	    ($data, $offsettosignm);
-	$self->{"signame"}=lc($signame) ."."; #dn_expand forgets trailing dots.
+	$self->{"signame"}=lc($signame) ;
 	my($sigmaterial)=substr($$data,$sigoffset,
 				($self->{"rdlength"}-$sigoffset+$offset));
 	$self->{"sigbin"}=$sigmaterial;
@@ -118,7 +125,7 @@ sub new_from_string {
 	$self->{"sigexpiration"}=  $sigexpiration;
 	$self->{"siginception"}= $siginception;
 	$self->{"keytag"}= $keytag;
-	$self->{"signame"}= lc($signame);
+	$self->{"signame"}= Net::DNS::stripdot(lc($signame));
 	$self->{"sig"}= $sig;
 	$self->{"sigbin"}= decode_base64($sig);
 	$self->{"vrfyerrstr"}="";
@@ -192,6 +199,7 @@ sub rr_rdata {
 	$rdata=$self->rr_rdata_without_sigbin;
 
 	if ($self->{"sig"} ne "NOTYETCALCULATED") {
+            $self->{"sigbin"}= decode_base64($self->{"sig"}) unless defined $self->{"sigbin"} ;
             $rdata .= $self->{"sigbin"};
 	}else{
             #do sigzero calculation based on current packet content...
@@ -425,7 +433,8 @@ sub create {
 
     if (!$sigzero)    {   
 	my  $labels=$datarrset->[0]->name;
-	$labels =~ s/\.^//;  # remove trailing dot.
+	$labels =~ s/\.$//;  # remove trailing dot.
+        $labels =~ s/^\*\.//;  # remove initial asterisk label
 	my @labels= split /\./ , $labels;
 	$self->{"labels"}= scalar(@labels);
 	
@@ -652,7 +661,7 @@ sub verify {
     }
     else                                  # Verifying other algorithms
     { 
-	$self->{"vrfyerrstr"}= "Algoritm ". $self->algorithm . " has not yet been implemented";
+	$self->{"vrfyerrstr"}= "Algorithm ". $self->algorithm . " has not yet been implemented";
 	return 0;
     }	
     
