@@ -1,5 +1,5 @@
 #
-# $Id: SEC.pm 359 2005-06-06 13:54:18Z olaf $
+# $Id: SEC.pm 1106 2013-08-23 09:17:11Z willem $
 #
 
 use strict;
@@ -12,9 +12,16 @@ use bytes;
 use Carp;
 use strict;
 use Exporter;
-use vars qw($VERSION @EXPORT_OK @ISA);
+use vars qw($SVNVERSION $VERSION $HAS_NSEC3 $HAS_DLV @EXPORT_OK @ISA);
 @ISA=qw(Exporter);
-$VERSION = '0.12_02';
+$VERSION = '0.16_01';
+
+$HAS_DLV=1;     # Signals availability of DLV to Net::DNS::RR
+$HAS_NSEC3=1;   # Signals availability of NSEC3 to Net::DNS::RR
+
+
+$SVNVERSION = (qw$LastChangedRevision: 1106 $)[1];
+
 
 @EXPORT_OK= qw (
               key_difference
@@ -37,7 +44,8 @@ other modules in this suite and a few functions that can be exported.
 =head1 DESCRIPTION
 
 The Net::DSN::SEC suite provides the resource records that are needed
-for DNSSEC (RFC 4033, 4034 and 4035). 
+for DNSSEC (RFC 4033, 4034 and 4035). In addition the DLV RR, a clone
+of the DS RR is supported (RFC 4431)
 
 It also provides support for SIG0. That later is useful for dynamic
 updates using key-pairs.
@@ -45,8 +53,9 @@ updates using key-pairs.
 RSA and DSA crypto routines are supported.
 
 For details see L<Net::DNS::RR::RRSIG>, L<Net::DNS::RR::DNSKEY>,
-L<Net::DNS::RR::NSEC> and L<Net::DNS::RR:DS>.  and see
-L<Net::DNS::RR::SIG> and L<Net::DNS::RR::KEY> for the use with SIG0.
+L<Net::DNS::RR::NSEC>, L<Net::DNS::RR:DS>, L<Net::DNS::RR::DLV>, and
+see L<Net::DNS::RR::SIG> and L<Net::DNS::RR::KEY> for the use with
+SIG0.
 
 Net::DNS contains all needed hooks to load the Net::DNS::SEC
 extensions when they are available.
@@ -74,6 +83,7 @@ Returns 0 on success or an error message on failure.
 =cut
 
 
+
 sub key_difference {
     my $a=shift;
     my $b=shift;
@@ -106,8 +116,8 @@ are not exported.
 
 =head2 algorithm
 
-    $value=Net::DNS::SEC->algorithm("RSA/SHA1");
-    $value=$self->algorithm("RSA/SHA1");
+    $value=Net::DNS::SEC->algorithm("RSASHA1");
+    $value=$self->algorithm("RSASHA1");
     $value=$self->algorithm(5);
 
     $algorithm=$self->algorithm();
@@ -123,6 +133,35 @@ mnemonic of the algorithm.
 
 Can also be called as a class method to do Mnemonic to Value conversion.
 
+=head2 digtype
+
+    $value=$self->digtype("SHA1");
+    $value=$self->digtype(1);
+
+    $algorithm=$self->digtype();
+    $memonic=$self->digtype("mnemonic");
+
+
+The digtype method is used to set or read the value of the digestype or
+hash algorithm field in Net::DNS::RR::DS and Net::DNS::RR::NSEC3
+objects.
+
+If supplied with an argument it will set the digetstype/hash algorithm
+accordingly, except when the argument equals the string "mnemonic" the
+method will return the mnemonic of the digetstype/hash algorithm.
+
+Can also be called as a class method to do Mnemonic to Value
+conversion, note however that it will then use the "Delegation Signer
+(DS) Resource Record (RR) Type Digest Algorithms" and not the "DNSSEC
+NSEC3 Hash Algorithms" IANA registry. If you want to specifically get
+access to the NSEC3  digest types then use a construct like:
+
+ bless $self, Net::DNS::RR::NSEC3;
+ $self->digtype("SHA1");
+
+
+
+
 =head1 COPYRIGHT
 
 Copyright (c) 2001-2005  RIPE NCC.  Author Olaf M. Kolkman <olaf@net-dns.org>
@@ -163,20 +202,30 @@ RFC4033, 4034 and 4035.
 
 
 
-sub algorithm {
+
+
+ sub algorithm {
     my $self=shift;
     my $argument=shift;
-    # classmethod is true if called as class method.
+
+   # classmethod is true if called as class method.
     my $classmethod=0;
     $classmethod=1 unless  ref ($self);
-
+ 
     my %algbyname = (
-		  "RSA/MD5"		=> 1,		
-#		  "DH"                  => 2,           # Not implemented
-		  "DSA"                 => 3,
-#		  "ECC"                 => 4,           # Not implemented
-		  "RSA/SHA1"            => 5,
-		  );
+	"RSAMD5"              => 1,
+	"DH"                  => 2,           # Not implemented
+	"DSA"                 => 3,
+	"ECC"                 => 4,           # Not implemented
+	"RSASHA1"             => 5,
+	"DSA-NSEC3-SHA1"      => 6,
+	"RSA-NSEC3-SHA1"      => 7,
+	"RSASHA256"           => 8,
+	"RSASHA512"           => 10,
+	"INDIRECT"            => 252,         # Not implemented
+	"PRIVATEDNS"          => 253,         # Not implemented
+	"PRIVATEOID"          => 254,         # Not implemented
+	);      
     my %algbyval = reverse %algbyname;
 
     # If the argument is undefined...
@@ -188,10 +237,9 @@ sub algorithm {
 
     # Argument has some value...
     $argument =~ s/\s//g; # Remove strings to be kind
-
+    $argument =~ s!RSA/!RSA!;  # Be kind for those who use RSA/SHA1
     if ($argument =~ /^\d+$/ ){    #Numeric argument.
-	carp "$argument does not map to a valid algorithm" unless 
-	    exists $algbyval{$argument};
+
 	if ($classmethod){
 	    return $argument ;
 	}else{
@@ -199,16 +247,14 @@ sub algorithm {
 	}
     }else{  # argument is not numeric
 	if ($classmethod){
-	    carp "$argument does not map to a valid algorithm" unless
-		exists $algbyname{uc($argument)};
+	    # This will return undefined if the argument does not exist
 	    return $algbyname{uc($argument)};
 	    
 	}else{ # Not a class method..
 	    if (lc($argument) eq "mnemonic"){
 		return $algbyval{$self->{"algorithm"}};
 	    }else{
-		carp "$argument does not map to a valid algorithm" unless
-		    exists $algbyname{uc($argument)};
+		# This will return undefined if the argument does not exist
 		return $self->{"algorithm"}=$algbyname{uc($argument)};
 	    }	    
 	}
@@ -226,4 +272,73 @@ sub algorithm {
 
 
 
+sub digtype {
+    _digtype(@_);
+}
+
+sub _digtype {
+    my $self=shift;
+    my $argument=shift;
+    # classmethod is true if called as class method.
+    my $classmethod=0;
+    $classmethod=1 unless  ref ($self);
+
+    my %digestbyname= (
+			"SHA1"		   => 1,		
+			"SHA256"	   => 2,		
+			);      
+
+    
+    if (! $classmethod && defined ($self->{'digestbyname'}) ){
+	%digestbyname= %{$self->{"digestbyname"}};
+    }
+
+
+    my %digestbyval = reverse %digestbyname;
+    
+    # If the argument is undefined...
+    
+    if (!defined $argument){
+	return if $classmethod;
+	return $self->{"digtype"};
+    }
+
+    # Argument has some value...
+    $argument =~ s/\s//g; # Remove strings to be kind
+
+    if ($argument =~ /^\d+$/ ){    #Numeric argument.
+	carp "$argument does not map to a valid digest type" unless 
+	    exists $digestbyval{$argument};
+	if ($classmethod){
+	    return $argument ;
+	}else{
+	    return $self->{"digtype"}=$argument ;
+	}
+    }else{  # argument is not numeric
+	if ($classmethod){
+	    carp "$argument does not map to a valid digest type" unless
+		exists $digestbyname{uc($argument)};
+	    return $digestbyname{uc($argument)};
+	    
+	}else{ # Not a class method..
+	    if (lc($argument) eq "mnemonic"){
+		return $digestbyval{$self->{"digtype"}};
+	    }else{
+		carp "$argument does not map to a valid digest type" unless
+		    exists $digestbyname{uc($argument)};
+		return $self->{"digtype"}=$digestbyname{uc($argument)};
+	    }	    
+	}
+
+	
+    }	
+    die "digtype method should never end here";
+
+	
+}
+
+
+
+
+
 
